@@ -1,6 +1,7 @@
 /* Declarations for retr.c.
    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation,
+   Inc.
 
 This file is part of GNU Wget.
 
@@ -43,7 +44,10 @@ extern bool output_stream_regular;
 /* Flags for fd_read_body. */
 enum {
   rb_read_exactly  = 1,
-  rb_skip_startpos = 2
+  rb_skip_startpos = 2,
+
+  /* Used by HTTP/HTTPS*/
+  rb_chunked_transfer_encoding = 4
 };
 
 int fd_read_body (int, FILE *, wgint, wgint, wgint *, wgint *, double *, int);
@@ -69,4 +73,6 @@ bool url_uses_proxy (struct url *);
 
 void set_local_file (const char **, const char *);
 
+bool input_file_url (const char *);
+
 #endif /* RETR_H */
